import json
from typing import Dict

from bravado.http_client import HttpClient
from bravado.http_future import FutureAdapter, HttpFuture
from bravado_core.response import IncomingResponse


class DjangoTestResponseAdapter(IncomingResponse):
    def __init__(self, django_response):
        self.django_response = django_response

    @property
    def status_code(self):
        return self.django_response.status_code

    @property
    def text(self):
        return self.django_response.content

    @property
    def raw_bytes(self):
        return self.django_response.content

    @property
    def reason(self):
        return self.django_response.content

    @property
    def headers(self):
        # convert django test response headers into a dict
        headers = dict((v[0], v[1]) for _, v in self.django_response._headers.items())

        # bravdo needs content type to be lower case to trigger validation
        headers["content-type"] = headers.get("Content-Type", None)
        return headers

    def json(self, **kwargs):
        return self.django_response.json()

    @property
    def data(self):
        return self.json()


class DjangoTestFutureAdapter(FutureAdapter):
    def __init__(self, client, request) -> None:
        self.client = client
        self.request = request

    def result(self, timeout):
        request = self.request
        request_method = getattr(self.client, request["method"].lower())
        url = request["url"]
        headers = request["headers"]

        # convert data from a string to json
        data = request.get("data", None)
        if data:
            data = json.loads(data)

        # handle get params
        params = request.get('params', {})

        # handle DRF and vanila Django seperately
        if self.client.__class__.__name__ == 'Client':
            response = request_method(url, data or params, **headers, content_type="application/json")
        else:
            response = request_method(url, data or params, **headers, format='json')

        return response


class DjangoTestHttpClient(HttpClient):
    def __init__(self, test_client, *args, **kwargs) -> None:
        self.test_client = test_client
        super().__init__(*args, **kwargs)

    def sanitize_params(self, request_params: Dict):
        sanitized_params = request_params.copy()

        if "connect_timeout" in sanitized_params:
            sanitized_params.pop("connect_timeout")

        if "timeout" in sanitized_params:
            sanitized_params.pop("timeout")

        return sanitized_params

    def request(self, request_params, operation=None, request_config=None):
        sanitized_params = self.sanitize_params(request_params)

        django_test_future = DjangoTestFutureAdapter(self.test_client, sanitized_params)

        return HttpFuture(django_test_future, DjangoTestResponseAdapter, operation, request_config)




# An example bravado config file to get the best results from bravado

config = {
    "response_metadata_class": "bravado.response.BravadoResponseMetadata",
    "disable_fallback_results": False,
    "validate_responses": True,
    "validate_requests": True,
    "validate_swagger_spec": True,
    "use_models": False,
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import re

from setuptools import setup


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)


def get_packages(package):
    """
    Return root package and all sub-packages.
    """
    return [dirpath
            for dirpath, dirnames, filenames in os.walk(package)
            if os.path.exists(os.path.join(dirpath, '__init__.py'))]


def get_package_data(package):
    """
    Return all files under the root package, that are not in a
    package themselves.
    """
    walk = [(dirpath.replace(package + os.sep, '', 1), filenames)
            for dirpath, dirnames, filenames in os.walk(package)
            if not os.path.exists(os.path.join(dirpath, '__init__.py'))]

    filepaths = []
    for base, filenames in walk:
        filepaths.extend([os.path.join(base, filename)
                          for filename in filenames])
    return {package: filepaths}


def get_long_description(long_description_file):
    """
    Read long description from file.
    """
    with open(long_description_file, encoding='utf-8') as f:
        long_description = f.read()

    return long_description


version = get_version('bravado_django_test_client')


setup(
    name='bravado_django_test_client',
    version=version,
    url='https://github.com/maximilianhurl/bravado-django-test-client',
    license='MIT',
    description='Bravado Django Test Client',
    long_description=get_long_description('README.md'),
    long_description_content_type='text/markdown',
    author='Max Hurl',
    author_email='max@maxhurl.co.uk',
    packages=get_packages('bravado_django_test_client'),
    package_data=get_package_data('bravado_django_test_client'),
    install_requires=[
        'django',
        'bravado',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Topic :: Internet :: WWW/HTTP',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
)
